/*----------------------------------------------------------------------------------*/
/*            1541-III, Hardware control routines, Written by Jan Derogee           */
/*----------------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------------*/

/*  History:                                                                        
    --------    
	2007-03-08	status of LED output was set to 1 (LED-ON) during config, must be 0 (LED-OFF)
	2006-06-22	changed baudrate to 115K2, this is more practical/compatible with the bootloader (currently)
	2006-06-21	added complete device configuration word, witout it the source is not complete
				in previous versions the user had to set the configuration word within MPLAB
				but to make this source independent, the configuration word must be included here!!
				and NOT in the development environment. Doing it within the source also documents it.
	2006-04-12	enabled interrupt on RB0 (for ATN handling under interrupt)
	2006-03-28	change in IO-pin definiions required for new PCB design
	2006-02-04	added timer-0 (for the led blinking routines)
	2006-02-02	improvements in code layout... i.o.w. making it more readable

*/

/*  TO DO:                                                                          
    ------      
*/

/*----------------------------------------------------------------------------------*/


/*--------------------------------------------------------*/
/*                        includes                        */
/*--------------------------------------------------------*/
#include <pic18.h>
#include <Lcd_Nokia_3310.h>
//#include <IEC_bus.h>
#include <hardware.h>

/*--------------------------------------------------------*/
/*            device configuration word (fuses)           */
/*--------------------------------------------------------*/

	/*configuration register for 18F2525 and 18F2620*/
	__CONFIG(1,IESODIS & HS & FCMDIS);
	__CONFIG(2,PWRTEN & BOREN & BORV28 & WDTPS32K & WDTDIS);
	__CONFIG(3,CCP2RB3 & PBANDIS & MCLREN & LPT1DIS);
	__CONFIG(4,STVREN & DEBUGDIS & LVPDIS & XINSTDIS);
	__CONFIG(5,UNPROTECT);
	__CONFIG(6,WRTEN);

/*--------------------------------------------------------*/
/*                         globals                        */
/*--------------------------------------------------------*/
bit	std_out;		/*this holds the destination of the putch- and printf-routines*/
/*--------------------------------------------------------*/

/*****************************************************************************************************************/
/*****************************************************************************************************************/

void HardwareInit(void)		/*initialize processor here*/
{
	PORTA=0x0D;		/*configure general I/O*/
	PORTB=0x00;		/*---------------------*/
	PORTC=0x44;		/*RC6(TXD)=1, RC2(CS)=1*/

	TRISA=0x3D;		/*in: RA0, RA2, RA3, RA4, RA5				out: all others*/
	TRISB=0x01;		/*in: RB0                                   out: all others*/
	TRISC=0x93;		/*in: RC7, RC4, RC1, RC0					out: all others*/

	ADCON0=0x01;	/*enable AD converter, select AN0 to be digitized*/
	ADCON1=0x0E;	/*no external references, only AN0 is configured as analog input*/
	ADCON2=0x86;	/*right justified, 0TAD, Tosc/64*/

					/*initialize SPI*/
	SMP = 0;		/*Input is valid in the middle of the clock*/
	CKE = 0;		/*Rising edge*/
	CKP = 1;		/*idle state for clock is high level*/
	SSPM3 = 0;		/*Speed f/64(312kHz), Master*/
	SSPM2 = 0;
	SSPM1 = 1;		
	SSPM0 = 0;
	SSPEN = 1;		/*SPI ON*/

	SPBRG=10;		/*initialize serial port for 10=115200, 129=9600BAUD @ 20MHz*/
	TXSTA=0x24;
	RCSTA=0x90;

	TMR0IE 	= 1;	/*enable interrupts for timer-0*/
	TMR0H	= 0;	/*clear the timer-value*/
	TMR0L	= 0;	/*clear the timer-value*/
	T08BIT	= 0;	/*select 16-bit mode*/
	T0CS	= 0;	/*select timer-mode*/
	PSA		= 0;	/*use prescaler*/
	T0PS2	= 1;	/*	256:1	timer-0 prescaler MSB*/
	T0PS1	= 1;	/*			timer-0 prescaler    */
	T0PS0	= 1;	/*			timer-0 prescaler LSB*/

	TMR1IE 	= 0;	/*disable interrupts for timer-1*/
	TMR1IP	= 1;	/*set to high priority*/
	TMR1H	= 0;	/*clear the timer-value*/
	TMR1L	= 0;	/*clear the timer-value*/
//	RD16	= 1;	/*select 16-bit mode*/
	T1RUN	= 1;	/*enable timer 1 oscillator*/
	T1CKPS1	= 0;	/*	  1:1	timer-1 prescaler MSB*/
	T1CKPS0	= 0;	/*			timer-1 prescaler    */
	T1OSCEN	= 0;	/*disable timer 1 inverter*/
	T1SYNC	= 1;	/*do not sync clock input*/
	TMR1CS	= 0;	/*select internal clock*/
	TMR1ON	= 0;	/*disable timer 1*/

	PR2 = 0xFF;		/*set PWM to a freq of 19.53KHz*/
	CCPR2L = 0;		/*set PWM output to 0*/
	T2CON = 0x04;	/*enable timer, postscale 1:1, prescale 1:1*/
	CCP2CON = 0x0c;	/*enable PWM mode*/


	INTEDG0	= 0;	/*set interrupt trigger to falling edge*/
	INT0IE	= 0;	/*disable interrupts on pin RB0 (INT0), this line is connected to the ATN line*/

	IPEN=1;			/*enable priority for interrrupts*/
	PEIE=1;
	GIE=1;
}

void Enable_ATN_interrupt(void)
{
	INT0IE	= 1;	/*enable interrupts on pin RB0 (INT0), this line is connected to the ATN line*/
}

/*redirect standard output to serial or LCD*/
void putch(unsigned char ch) 
{
	switch (std_out)
	{
		case RS232:
		{
			while(TRMT==0);
			TXREG=ch;  	
			break;
		}

		case LCD:
		{
			Lcd_char(ch);
			break;
		}

		default:
		{
			while(TRMT==0);
			TXREG=ch;  	
			break;
		}
	}
}

/*SPI-bus*/
unsigned char SPI(unsigned char d)		
{
	SSPBUF = d;
	while (!BF);			/*Wait untill controller is ready*/
	return(SSPBUF);			/*Return with received value*/
}

	

	
/*
The author of sourcecode without comments is the only one to fully understand the written code... for the first 3 months.
*/
